/*
 *  Copyright (c) 2007 Jiri Benc <jbenc@upir.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>
#include <hildon-widgets/hildon-banner.h>
#include <dbus/dbus.h>
#include "cmdnav.h"
#include "run.h"

/* #define DEBUG */

#ifdef DEBUG
#define debug_print	printf
#else
#define debug_print(...)
#endif

static void run_xterm(struct cmd *cmd)
{
	char *run, *tmp = NULL;
	DBusConnection *bus;
	DBusMessage *msg;

	bus = dbus_bus_get(DBUS_BUS_SESSION, NULL);
	if (!bus)
		return;
	msg = dbus_message_new_method_call("com.nokia.xterm", "/com/nokia/xterm",
		"com.nokia.xterm", "run_command");
	if (!msg)
		return;

	run = cmd->cmd;
	if (cmd->flags & CMD_FLAG_ROOT)
		run = tmp = g_strjoin(NULL, "sudo ", run, NULL);
	run = g_strescape(run, NULL);
	if (tmp)
		g_free(tmp);
	tmp = run;
	run = g_strjoin(NULL, "sh -c \"", run, " ; read x\"", NULL);
	g_free(tmp);
	debug_print("running: %s\n", run);
	dbus_message_append_args(msg, DBUS_TYPE_STRING, &run, DBUS_TYPE_INVALID);
	dbus_message_set_no_reply(msg, TRUE);
	dbus_connection_send(bus, msg, NULL);
	/* dbus_connection_flush(bus); */
	g_free(run);
	dbus_message_unref(msg);
	dbus_connection_unref(bus);
}

void run_cmd(struct cmd *cmd)
{
	pid_t pid;
	char *run;
	char buf[64];

	if (cmd->flags & CMD_FLAG_XTERM) {
		run_xterm(cmd);
		return;
	}

	pid = fork();
	if (pid < 0) {
		hildon_banner_show_information(NULL, "qgn_note_gene_syserror", "Execution failed");
		return;
	}
	if (pid > 0) {
		g_snprintf(buf, 64, "%s started", cmd->name);
		hildon_banner_show_information(NULL, NULL, buf);
		return;
	}

	run = cmd->cmd;
	if (cmd->flags & CMD_FLAG_ROOT)
		run = g_strjoin(NULL, "sudo ", run, NULL);

	debug_print("running: %s\n", run);
	execl("/bin/sh", "/bin/sh", "-c", run, NULL);
	/* TODO: send a warning to parent */
	exit(0);
}

void run_prefs(void)
{
	DBusConnection *bus;
	DBusMessage *msg;
	const char *tmp1;
	dbus_bool_t tmp2;

	bus = dbus_bus_get(DBUS_BUS_SESSION, NULL);
	if (!bus)
		return;
	msg = dbus_message_new_method_call("com.nokia.controlpanel", "/com/nokia/controlpanel/rpc",
		"com.nokia.controlpanel", "run_applet");
	if (!msg)
		return;
	tmp1 = "libcmd-navigator-prefs.so";
	tmp2 = 1;
	debug_print("starting preferences\n");
	dbus_message_append_args(msg, DBUS_TYPE_STRING, &tmp1,
		DBUS_TYPE_BOOLEAN, &tmp2, DBUS_TYPE_INVALID);
	dbus_message_set_no_reply(msg, TRUE);
	dbus_connection_send(bus, msg, NULL);
	/* dbus_connection_flush(bus); */
	dbus_message_unref(msg);
	dbus_connection_unref(bus);
}

void dezombie()
{
	int status;

	while (waitpid(-1, &status, WNOHANG) > 0)
		;
}
